
''*********************************************
''*  Scribbler Default Program                *
''*  Author: Ben Wirz, Element Products, Inc. *
''*  Copyright (c) 2010 Parallax, Inc.        *  
''*  See end of file for terms of use.        *
''********************************************* 
''
''The Scribbler comes with a series of a 8 Demonstration
''Programs installed when purchased new.  The programs
''are ordered such that each of the Scribbler's sensing
''capabilities are shown statically followed by an
''interactive demonstration of the sensor's
''capabilities.  The demonstration programs are
''selected by by pushing the Blue Push Switch 1 to 8
''times following power-on or by utilizing the
''finger selection technique.  The 3 user LED's
''sequence with each  switch push to indicate the currently
''selected program.  A new program can be selected at any
''time by pushing the switch - the scribbler does not need
''to be turned off.
''
''1 Button Push    : Light Sensor Display
''2 Button Pushes  : Light Follow Program
''3 Button Pushes  : Obstacle Sensor Display
''4 Button Pushes  : Obstacle Avoid Program
''5 Button Pushes  : Line Sensor Display
''6 Button Pushes  : Line Follow Program
''7 Button Pushes  : Drawing Program
''8 Button Pushes  : Ambulance Program  
''
''(See the individual functions and the Scribbler Quick
''Start Guide for additional documentation.)
''
''Power Indicator LED:
''  
''The Blue LED serves as a power-on indicator as
''well as displaying the battery condition.
''
'' Solid Blue       : Battery Fully Charged
'' Dim Blue         : Battery Partially Charged
'' Blinking Blue    : Battery Low / Replace or Recharge
''
''Over Current Condition:
''
''In the case of a motor over current condition due
''to an electrical short circuit, the motor driver
''will be disabled in hardware.  This condition is
''indicated by all three of the Red LED's Blinking
''and the Blue LED being turned off.  To reset the
''condition, cycle the Scribblers's power off for 15 seconds
''and then back on.
''
''Scribbler Test Utility:
''
''A Serial Terminal Test Utility has been included with
''the default program for testing the Scribbler.  The utility
''includes a factory test and several debugging
''programs.  The utility was designed to be used with
''a host PC running the Parallax Serial Terminal program.
''
''Parallax Serial Terminal Program Setup
''
''Com Port:  Choose the correct port for the Scribbler
''Baud Rate:  19,200bps
''Click Restore Defaults in the Preferences Window 
''The DTR, RTS & Echo On Boxes should be unchecked
'' 
''With the Parallax Serial Terminal enabled on the
''host PC, hold down the blue button while turning on
''the Scribbler's power switch to start the utility.  To exit
''the Test Utility, cycle the Scribbler's power switch.

CON

   VERSION       = 2                                    'Release Version Number
   SUBVERSION    = 2579                                 'Internal Version Number    
           
  'Clock setup
  _clkmode      = xtal1 + pll16x
  _xinfreq      = 5_000_000

  'Low level hardware parameters                                                
  LINE_TRIP                 = 40                        'The maximum line sensor reading
                                                        'for a black line.
  LINE_TRIP_ALT             = 30                        'The alternative maximum line sensor black line.
                                      
  LINE_LIFT                 = 7                         'The maximum line sensor reading for
                                                        'detecting a lifted robot.

  LIGHT_TRIP_INC            = 10                        'The minimum delta value change in the light sensor
                                                        'output over the running average for a finger
                                                        'detection.  

  OBS_THLD                  = 20                        'The obstacle sensor sensitivity setting. (1 to 100)
  OBS_MAX_SAMPLE_RATE       = 8                         'The obstacle sensor maximum sample rate in Hz. 
  PERIODIC_FREQ             = OBS_MAX_SAMPLE_RATE       'The frequency of the periodic function called by a repeat loop in Hz.  
  
  MOTOR_OC_TRIP             = 210                       'The minimum motor driver over current detect level

                                                        'S2 battery voltage trip levels. (NiMH / Alkaline)
  S2_BATTERY_LOW_TRIP       = (700*2550)/(400*33)       '7.0V
  S2_BATTERY_DEAD_TRIP      = (600*2550)/(400*33)       '6.0V
   
                                                        'S3 battery voltage trip levels. (LiPo)
  S3_BATTERY_LOW_TRIP       = (350*2550)/(200*33)       '3.5V
  S3_BATTERY_DEAD_TRIP      = (320*2550)/(200*33)       '3.2V
  S3_BATTERY_OFF_TRIP       = (300*2550)/(200*33)       '3.0V 
   
                                                        'Global feature enable / disable options.
  STALL_CHECK               = TRUE                      'Enable stall detection?
  OBSTACLE_CHECK            = TRUE                      'Enable obstacle detection? 
  SONG_ENABLE               = TRUE                      'Enable song playing at the start of programs? 
                                                        '(does not fully disable the speaker)

OBJ
' scribbler            : "scribbler"
  scribbler_test       : "scribbler_test"
' music                : "scribbler_music"
     
VAR

  BYTE state_current,state_previous,state_changed,battery_min,light_last,light_last_detect,light_running_avg
  LONG periodic_next_cnt

  BYTE obs_sample_right, obs_sample_left                'The sampled obstacle sensor readings.
  BYTE obs_sample_side                                  'The current obstacle sensor side being sampled..
  
PUB start | demo_select
  ''Scribbler Demo Start-up & Program Selection

  
  scribbler_test.start(VERSION,SUBVERSION,LINE_TRIP_ALT,OBS_THLD) 

{{
+------------------------------------------------------------------------------------------------------------------------------+
|                                                   TERMS OF USE: MIT License                                                  | 
+------------------------------------------------------------------------------------------------------------------------------+
|Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation    | 
|files (the "Software"), to deal in the Software without restriction, including without limitation the rights to use, copy,    |
|modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, and to permit persons to whom the Software|
|is furnished to do so, subject to the following conditions:                                                                   |
|                                                                                                                              |
|The above copyright notice and this permission notice shall be included in all copies or substantial portions of the Software.|
|                                                                                                                              |
|THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE          |
|WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR         |
|COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,   |
|ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.                         |
+------------------------------------------------------------------------------------------------------------------------------+
}}
